
# q-expansion of newform 54.8.a.g, downloaded from the LMFDB on 03 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 54
weight = 8
poly_data = [-82, -1, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0], 1], [[-27, 54], 1]]

hecke_ring_character_values = None
aps_data = [[-8, 0], [0, 0], [24, 1], [440, 1], [-3615, -8], [4280, 20], [-12852, 20], [18524, -70], [29814, 20], [137640, -230], [122792, -285], [-35530, 868], [-247260, -14], [412640, -318], [463854, -840], [691056, -245], [-794460, 1016], [-1510984, 240], [-941080, 1842], [1697220, -5860], [447545, 4824], [-184960, 3320], [4676025, 3240], [456480, 9314], [679895, -14776], [2180280, -15217], [11760080, 5268], [-5326515, -18680], [-1381210, -41820], [-11303004, -23550], [-41246800, -4089], [-524595, 42376], [-4743096, -46910], [-16827304, -78500], [37804560, -65863], [16499024, -93225], [9707540, -89064], [-38336740, -95456], [25798074, 46800], [12419688, 75405], [17082315, -133640], [-15857872, -124160], [-74877360, -161980], [100764725, 68880], [110676960, 25835], [90847568, 73535], [78629396, -76870], [-136756360, -9408], [-132331092, -154920], [5741582, -351480], [-81913968, -373490], [-86476710, 71364], [-181213714, 96040], [-167952960, 336448], [-21073404, 901200], [-103140186, 61100], [384277680, 38890], [49745720, -256575], [181239920, -6216], [337427940, 510820], [-48648460, -482442], [-480800928, 264110], [564218660, -420660], [-584852610, 587788], [258323165, 559000], [11143416, 545255], [217089800, -370150], [506844050, -1339016], [633973119, -1049640], [-1392799726, -34340], [-451192092, -1829930], [1176433290, 67992], [301727240, 2149671], [-970144780, 735500], [518010464, -1114340], [847585428, 2159860], [563006640, 342113], [44653160, -236388], [-531147360, -1658924], [-149548369, -1151840], [1761751500, -1710096], [1567771136, 2019880], [-1324210590, -2861104], [-951149275, -1380600], [1573513496, 103155], [109723476, 4472400], [-838563660, 2164062], [1420772975, 1500216], [-1885223760, -743783], [-2134535080, -3337003], [2520129753, -3089840], [758365950, -3765816], [723277280, -7416016], [-2301443805, -3848584], [208304600, 7250850], [-728197110, -4522860], [-278555880, -1864061], [-683773740, 8826632], [-4218403660, -315588], [299459180, -687560], [1386579860, 8035044], [4582583424, 3105785], [2576510583, -3411000], [-3013230120, -6163948], [1892567876, -6245360], [-320648290, 6611784], [1233059601, 2606600], [-2433769488, -9195970], [-3259771410, 11882168], [721636307, 852840], [389774480, 14074192], [-2355156250, -9307008], [2694676236, -2637170], [5851945988, 8869320], [2179654760, -7788455], [-4210134180, 8261626], [-4240150240, 13737780], [-2576307384, 683640], [-259452840, 21142085], [-2329609065, 1835792], [7778941898, -4769460], [-10214091235, 5145008], [3083045448, 7235990], [5512602780, 14397840], [11397406400, 4762980], [-734100120, -20181125], [-6345865972, -13394320], [659078820, 17818176], [6548117120, -18574839], [1467436550, 23039148], [2765897804, 6320100], [7581219504, 3072540], [5318943440, 29219335], [2503500050, 13452168], [-6746403000, 23295816], [-2091565759, 3629600], [13506928704, 16666410], [-8438264320, 19938144], [18044723904, 4801485], [-19226957760, 372062], [18700965236, -4884750], [18772819680, -6279694], [-11293149640, -14157121], [-1974799008, -22346080], [15627714482, -6375400], [-9060340020, 32764524], [-23309949610, -11539992], [10653283236, 16771500], [15795460760, 20177200], [-10606378644, -25795600], [-10223154670, 930592], [-10062690660, -30093080], [-3603103180, 53382626], [9956551848, 50028580], [32486839460, -6711094], [5519510310, 16684032], [-12446227744, -20901285], [13696093560, -5799660], [-2590854325, -36500504], [-6810362760, -47096467], [-1292287281, -59267040], [-10166768424, 58501740], [-20446246840, -67788327], [-17161534935, 65618232], [-19581479904, -10224310], [34100527278, -7178200], [40674713672, 19835385], [-4069581040, 7676772]]
